function [T]=get_propagation_matrices(e,part,int,forward)
%@LPM\GET_PROPAGATION_MATRICES
%[T]=get_propagation_matrices(e,part,int,forward)
% returns the propagation matrices

%opm: eigenlijk is asymptotische sneller dan analytische omdat daar geen
%evaluaties van airy-functies moeten gebeuren, nochtans wordt bij
%asymptotische niets vooraf berekend
if isnan(e)
    error('OVERFLOW')
end
hm=part.h;
cum=part.cu;
cupm=part.cup;
cvm=part.cv;
cvpm=part.cvp;
vb0=part.V0;
v=part.V;

for i=1:length(int)
tt=int(i);
h=hm(tt);   
f0=vb0(tt)-v(1,tt)*h;
f1=2*v(1,tt);
f3=abs(f1)^(1/3);
if f1<0
    f3=-f3;
end
f0e=f0-e;
t0=f0e/f3^(2);
t=f3*h+t0;

asymp=0;
if f1 == 0
    if f0e<0
          asymp=-1;
          srf0=sqrt(-f0e);
          cf0=cos(srf0*h);
          sf0=-srf0*sin(srf0*h);
    else
          asymp=1;
          srf0=sqrt(f0e);
          cf0=cosh(srf0*h);
          sf0=srf0*sinh(srf0*h);
    end
else
   if t0<-2 && t<-2
       asymp=-1;
       srf0=sqrt(-f0e);
       cf0=cos(srf0*h);
       sf0=-srf0*sin(srf0*h);
   elseif t0>2 && t>2
       asymp=1;
       srf0=sqrt(f0e);
       cf0=cosh(srf0*h);
       sf0=srf0*sinh(srf0*h);
   end
end
 
% asymp=1;
% if t0<-1 && t<-1
%        asymp=-1;
%        cf0=cos(sqrt(-f0e)*h);
%        sf0=-sqrt(-f0e)*sin(sqrt(-f0e)*h);
%    elseif t0>1 && t>1
%        asymp=1;
%        cf0=cosh(sqrt(f0e)*h);
%        sf0=sqrt(f0e)*sinh(sqrt(f0e)*h);
% end
% asymp=0
%    
if asymp==0 %analytic form
    cu=cum(:,:,tt);
    cv=cvm(:,:,tt);
    cup=cupm(:,:,tt);
    cvp=cvpm(:,:,tt);
       %zeroth order analytic
     [ai0,ierr] = airy(0,t0);
     if abs(imag(ai0))<4*eps
         ai0=real(ai0);
     end
     if ierr>0
         disp('Airy functions not accurate')
     end
     dai0 = real(airy(1,t0));
     bi0 = real(airy(2,t0));
     dbi0 = real(airy(3,t0));
     ai = real(airy(0,t));
     dai = real(airy(1,t));
     bi = real(airy(2,t));
     dbi = real(airy(3,t));
     u0=pi*(dbi0*ai-dai0*bi);
     u0p=pi*f3*(dbi0*dai-dai0*dbi);
     v0=pi*(-bi0*ai+ai0*bi)/f3;
     v0p=pi*(-bi0*dai+ai0*dbi);


     q=(f0e/f1).^(0:10);
     q=q';
     su1=cu(1,:)*q;
     su2=cu(2,:)*q;
     su3=cu(3,:)*q;
     su4=cu(4,:)*q;
     sv1=cv(1,:)*q;
     sv2=cv(2,:)*q;
     sv3=cv(3,:)*q;
     sv4=cv(4,:)*q;
     sup1=cup(1,:)*q;
     sup2=cup(2,:)*q;
     sup3=cup(3,:)*q;
     sup4=cup(4,:)*q;
     svp1=cvp(1,:)*q;
     svp2=cvp(2,:)*q;
     svp3=cvp(3,:)*q;
     svp4=cvp(4,:)*q;
     
     pu = u0+su1*u0+su2*v0+su3*u0p+su4*v0p;
     pv = v0+sv1*u0+sv2*v0+sv3*u0p+sv4*v0p;
     pup = u0p+sup1*u0+sup2*v0+sup3*u0p+sup4*v0p;
     pvp = v0p+svp1*u0+svp2*v0+svp3*u0p+svp4*v0p;
    
%      f123=f1^(2/3);
%      u0e=pi*(-dai*dbi0/f123-ai*f0e*bi0/f1^(4/3)+dbi*dai0/f123+bi*f0e*ai0/f1^(4/3));
%      u0pe=pi*f3*(-(h+f0e/f1)*ai*dbi0/f3-dai*f0e*bi0/f1^(4/3)+(h+f0e/f1)*bi*dai0/f3+dbi*f0e*ai0/f1^(4/3));
%      v0e=pi*(dai*bi0/f123+ai*dbi0/f123-dbi*ai0/f123-bi*dai0/f123)/f3;
%      v0pe=pi*((h+f0e/f1)*ai*bi0/f3+dai*dbi0/f123-(h+f0e/f1)*bi*ai0/f3-dbi*dai0/f123);
%      
%      q=(f0e/f1).^(1:10);
%      pue = u0e+su1*u0e+su2*v0e+su3*u0pe+su4*v0pe+sum(-cu(1,2:11).*q.*(1:10))/f0e*u0+...
%          sum(-cu(2,2:11).*q.*(1:10))/f0e*v0+sum(-cu(3,2:11).*q.*(1:10))/f0e*u0p+...
%          sum(-cu(4,2:11).*q.*(1:10))/f0e*v0p;
%      pve = v0e+sv1*u0e+sv2*v0e+sv3*u0pe+sv4*v0pe+sum(-cv(1,2:11).*q.*(1:10))/f0e*u0+...
%          sum(-cv(2,2:11).*q.*(1:10))/f0e*v0+sum(-cv(3,2:11).*q.*(1:10))/f0e*u0p+...
%          sum(-cv(4,2:11).*q.*(1:10))/f0e*v0p;
%      puep = u0pe+sup1*u0e+sup2*v0e+sup3*u0pe+sup4*v0pe+sum(-cup(1,2:11).*q.*(1:10))/f0e*u0+...
%          sum(-cup(2,2:11).*q.*(1:10))/f0e*v0+sum(-cup(3,2:11).*q.*(1:10))/f0e*u0p+...
%          sum(-cup(4,2:11).*q.*(1:10))/f0e*v0p;
%      pvep = v0pe+svp1*u0e+svp2*v0e+svp3*u0pe+svp4*v0pe+sum(-cvp(1,2:11).*q.*(1:10))/f0e*u0+...
%          sum(-cvp(2,2:11).*q.*(1:10))/f0e*v0+sum(-cvp(3,2:11).*q.*(1:10))/f0e*u0p+...
%          sum(-cvp(4,2:11).*q.*(1:10))/f0e*v0p;
   
else %asymptotic forms
  %  disp('asymptotic')
    h2=h*h;
    h3=h*h2;
    h4=h*h3;
    h5=h*h4;
    h6=h*h5;
    h7=h*h6;
    h8=h*h7;
    h9=h*h8;
    h10=h*h9;
    h11=h*h10;
    h12=h*h11;
    h13=h*h12;
    h14=h*h13;
    h15=h*h14;
    f0e2=f0e*f0e;
    f0e3=f0e*f0e2;
    f0e4=f0e*f0e3;
    f0e5=f0e*f0e4;
    f0e6=f0e*f0e5;
    f0e7=f0e*f0e6;
    f12=f1*f1;
    f13=f1*f12;
    f14=f1*f13;
    f15=f1*f14;

   %zeroth order correction
     u0a= cf0+(-f0e*cf0*h+sf0+f0e*h2*sf0)*f1/f0e2/4+(3*f0e2*h4*cf0-10*f0e*h3*sf0+...
         21*f0e*h2*cf0-21*h*sf0)*f12/f0e3/96+(105*sf0-70*f0e2*h3*cf0+...
         105*f0e*h2*sf0+28*f0e2*h4*sf0-7*f0e3*h5*cf0+f0e3*h6*sf0-105*f0e*cf0*h)*f13/f0e5/384+...
         (238*f0e3*h6*cf0-12285*h*sf0+3*f0e4*h8*cf0+3255*f0e2*h4*cf0-1050*f0e2*h5*sf0-...
         36*f0e3*h7*sf0-7350*f0e*h3*sf0+12285*f0e*h2*cf0)*f14/f0e6/18432;

      v0a= sf0/f0e+(f0e*h2*cf0-h*sf0)*f1/f0e2/4+(15*sf0-10*f0e2*h3*cf0+15*f0e*h2*sf0+...
          3*f0e2*h4*sf0-15*f0e*cf0*h)*f12/f0e4/96+(f0e3*h6*cf0-7*f0e2*h5*sf0+25*f0e2*h4*cf0-...
          60*f0e*h3*sf0+105*f0e*h2*cf0-105*h*sf0)*f13/f0e5/384+...
          (-6930*f0e2*h3*cf0-10395*f0e*cf0*h-966*f0e3*h5*cf0+226*f0e3*h6*sf0-...
          36*f0e4*h7*cf0+3045*f0e2*h4*sf0+10395*f0e*h2*sf0+10395*sf0+3*f0e4*h8*sf0)*f14/f0e7/18432;

      du0a=sf0+(f0e*h2*cf0+h*sf0)*f1/f0e/4+(-21*sf0+3*f0e2*h4*sf0+21*f0e*cf0*h-9*f0e*h2*sf0+...
          2*f0e2*h3*cf0)*f12/f0e3/96+(f0e3*h6*cf0-f0e2*h5*sf0-7*f0e2*h4*cf0+42*f0e*h3*sf0-...
          105*f0e*h2*cf0+105*h*sf0)*f13/f0e4/384+(-12285*sf0-14*f0e3*h6*sf0+...
          3*f0e4*h8*sf0-1995*f0e2*h4*sf0+5670*f0e2*h3*cf0-9765*f0e*h2*sf0+...
          12285*f0e*cf0*h-12*f0e4*h7*cf0+378*f0e3*h5*cf0)*f14/f0e6/18432;

      dv0a=cf0+(-sf0+f0e*cf0*h+f0e*h2*sf0)*f1/f0e2/4+(3*f0e2*h4*cf0+2*f0e*h3*sf0-...
          15*f0e*h2*cf0+15*h*sf0)*f12/f0e3/96+(105*f0e*cf0*h-f0e3*h5*cf0+f0e3*h6*sf0-...
          105*sf0-75*f0e*h2*sf0+40*f0e2*h3*cf0-10*f0e2*h4*sf0)*f13/f0e5/384+...
          (5250*f0e*h3*sf0-10395*f0e*h2*cf0+3*f0e4*h8*cf0-12*f0e3*h7*sf0-26*f0e3*h6*cf0+...
          10395*h*sf0-1785*f0e2*h4*cf0+390*f0e2*h5*sf0)*f14/f0e6/18432;
      
      %first order correction

	f0inv=1/f0e;
	f0inv2=f0inv^2;
	f0inv3=f0inv*f0inv2;
	f0inv4=f0inv*f0inv3;
	f0inv5=f0inv*f0inv4;
	f0inv6=f0inv*f0inv5;
	f0inv7=f0inv*f0inv6;
    f0inv8=f0inv*f0inv7;
	qinv=f1/f0e;
	qinv2=qinv^2;
	qinv3=qinv^3;
	qinv4=qinv^4;
    v2=v(2,tt);
    v3=v(3,tt);
    v4=v(4,tt);

      s1 = 15/2*f0inv3*v3*sf0+(3*h2*sf0-15/2*h*cf0)*v3*f0inv2-f0inv*v3*h3*cf0/2;
      s2 = s1+(-2205/8*f0inv4*v4*sf0+(-15/4*v2*sf0-45/4*v3*h*sf0+(-975/8*h2*sf0+2205/8*h*cf0)*v4)*f0inv3+...
          ((-3/2*h2*sf0+15/4*h*cf0)*v2+(-39/8*h3*sf0+45/4*h2*cf0)*v3+(-33/8*h4*sf0+30*h3*cf0)*v4)*f0inv2+...
          (v2*h3*cf0/4+(-h5*sf0/8+9/8*h4*cf0)*v3+v4*h5*cf0/4)*f0inv)*qinv;
      s3 = s2+((7245/64*v3*sf0+21735/32*v4*h*sf0)*f0inv4+(231/32*v2*h*sf0+(1995/32*h2*sf0-7245/64*h*cf0)*v3+...
          (4935/16*h3*sf0-21735/32*h2*cf0)*v4)*f0inv3+((-231/32*h2*cf0+99/32*h3*sf0)*v2+...
          (115/16*h4*sf0-1575/64*h3*cf0)*v3+(435/32*h5*sf0-2625/32*h4*cf0)*v4)*f0inv2+...
          ((-11/16*h4*cf0+11/160*h5*sf0)*v2+(5/24*h6*sf0-3/2*h5*cf0)*v3+(h7*sf0/16-43/32*h6*cf0)*v4)*f0inv-...
          v3*h7*cf0/64)*qinv2;
      s6 = (-2027025/256*f0inv5*v4*sf0+(-693/16*v2*sf0-43065/128*v3*h*sf0+(2027025/256*h*cf0-...
          1202355/256*h2*sf0)*v4)*f0inv4+((-1827/64*h2*sf0+693/16*h*cf0)*v2+...
          (-41985/256*h3*sf0+43065/128*h2*cf0)*v3+(65835/32*h3*cf0-170415/256*h4*sf0)*v4)*f0inv3+...
          ((903/64*h3*cf0-77/16*h4*sf0)*v2+(-3035/256*h5*sf0+13275/256*h4*cf0)*v3+(9975/64*h5*cf0-...
          3325/128*h6*sf0)*v4)*f0inv2+((343/320*h5*cf0-139/960*h6*sf0)*v2+...
          (131/64*h6*cf0-239/896*h7*sf0)*v3+(191/64*h7*cf0-167/768*h8*sf0)*v4)*f0inv+3/320*h7*v2*cf0+...
          (-h9*sf0/768+19/768*h8*cf0)*v3+v4*h9*cf0/128)*qinv3;
      s9 = (32207175/1024*v4*h*sf0+11550825/4096*v3*sf0)*f0inv5+(153153/1024*v2*h*sf0+...
          (-11550825/4096*h*cf0+3906045/2048*h2*sf0)*v3+(-32207175/1024*h2*cf0+...
          255255/16*h3*sf0)*v4)*f0inv4+((79695/1024*h3*sf0-153153/1024*h2*cf0)*v2+...
          (-3961815/4096*h3*cf0+45375/128*h4*sf0)*v3+(1435665/1024*h5*sf0-5600595/1024*h4*cf0)*v4)*f0inv3+...
          ((7665/1024*h5*sf0-7161/256*h4*cf0)*v2+(-24255/256*h5*cf0+4855/256*h6*sf0)*v3+(-284515/1024*h6*cf0+...
          21805/512*h7*sf0)*v4)*f0inv2;
      s8 = s9+((-3801/2560*h6*cf0+1603/7680*h7*sf0)*v2+(29767/86016*h8*sf0-83399/28672*h7*cf0)*v3+...
          (-2533/512*h8*cf0+3839/9216*h9*sf0)*v4)*f0inv+(13/15360*h9*sf0-29/1536*h8*cf0)*v2+...
          (-8185/258048*h9*cf0+13/6144*h10*sf0)*v3+(-71/3072*h10*cf0+h11*sf0/1536)*v4-...
          1/f0inv*v3*h11*cf0/12288;
      u1a = s3+s6+s8*qinv4;


      s1 = 105/2*f0inv3*v4*sf0+(3/2*v2*sf0+(45/2*h2*sf0-105/2*h*cf0)*v4)*f0inv2+...
          (-5*v4*h3*cf0-3/2*v2*h*cf0)*f0inv+f0inv*v2*sf0*h2/2;
      s2 = s1+f0inv*v4*h4*sf0/2+((-525/8*v4*h*sf0-30*v3*sf0)*f0inv3+...
          (-9/8*v2*h*sf0+(30*h*cf0-51/4*h2*sf0)*v3+(-30*h3*sf0+525/8*h2*cf0)*v4)*f0inv2+...
          ((-h3*sf0/2+9/8*h2*cf0)*v2+(-h4*sf0/4+11/4*h3*cf0)*v3+(-11/8*h5*sf0+65/8*h4*cf0)*v4)*f0inv+...
          h4*v2*cf0/8+h6*v4*cf0/8)*qinv;
      s3 = s2+(94185/64*f0inv4*v4*sf0+(777/64*v2*sf0+105/2*v3*h*sf0+...
          (-94185/64*h*cf0+46305/64*h2*sf0)*v4)*f0inv3+((377/64*h2*sf0-777/64*h*cf0)*v2+...
          (375/16*h3*sf0-105/2*h2*cf0)*v3+(-7455/32*h3*cf0+3555/64*h4*sf0)*v4)*f0inv2+...
          ((-59/32*h3*cf0+7/16*h4*sf0)*v2+(-95/16*h4*cf0+7/8*h5*sf0)*v3+...
          (-339/32*h5*cf0+161/96*h6*sf0)*v4)*f0inv+(h6*sf0/64-89/960*h5*cf0)*v2-h6*v3*cf0/16+(-5/24*h7*cf0...
          +h8*sf0/64)*v4)*qinv2;
      s6 = ((-70785/128*v3*sf0-1036035/256*v4*h*sf0)*f0inv4+(-6993/256*v2*h*sf0+...
          (70785/128*h*cf0-39915/128*h2*sf0)*v3+(1036035/256*h2*cf0-30555/16*h3*sf0)*v4)*f0inv3+...
          ((-1603/128*h3*sf0+6993/256*h2*cf0)*v2+(-4975/128*h4*sf0+255/2*h3*cf0)*v3+...
          (-29295/256*h5*sf0+143535/256*h4*cf0)*v4)*f0inv2+((875/256*h4*cf0-79/128*h5*sf0)*v2+...
          (-131/96*h6*sf0+277/32*h5*cf0)*v3+(-13/6*h7*sf0+4473/256*h6*cf0)*v4)*f0inv+...
          (-11/960*h7*sf0+163/1920*h6*cf0)*v2+(-h8*sf0/128+191/1344*h7*cf0)*v3+...
          (91/384*h8*cf0-17/768*h9*sf0)*v4+(h8*v2*cf0/768+v4*h10*cf0/768)/f0inv)*qinv3;
      s9 = 209684475/4096*f0inv5*v4*sf0+(837837/4096*v2*sf0+920205/512*v3*h*sf0+...
          (-209684475/4096*h*cf0+125180055/4096*h2*sf0)*v4)*f0inv4+((538923/4096*h2*sf0-...
          837837/4096*h*cf0)*v2+(227205/256*h3*sf0-920205/512*h2*cf0)*v3+...
          (18264015/4096*h4*sf0-27642615/2048*h3*cf0)*v4)*f0inv3+...
          ((-64911/1024*h3*cf0+11193/512*h4*sf0)*v2+(35265/512*h5*sf0-147675/512*h4*cf0)*v3+...
          (207725/1024*h6*sf0-1123815/1024*h5*cf0)*v4)*f0inv2;
      s8 = s9+((54719/61440*h6*sf0-108073/20480*h5*cf0)*v2+(349/192*h7*sf0-6489/512*h6*cf0)*v3+...
          (6517/2048*h8*sf0-7329/256*h7*cf0)*v4)*f0inv+(1919/184320*h8*sf0-1649/15360*h7*cf0)*v2+...
          (-1069/5376*h8*cf0+55/3584*h9*sf0)*v3+(-10913/36864*h9*cf0+913/36864*h10*sf0)*v4+...
          ((h10*sf0/12288-21/20480*h9*cf0)*v2-v3*h10*cf0/1536+(-11/6144*h11*cf0+h12*sf0/12288)*v4)/f0inv;
      du1a = s3+s6+s8*qinv4;


      s1 = -105/2*f0inv4*v4*sf0+(-3/2*v2*sf0+(-45/2*h2*sf0+...
          105/2*h*cf0)*v4)*f0inv3+((-h2*sf0/2+3/2*h*cf0)*v2+(-h4*sf0/2+5*h3*cf0)*...
          v4)*f0inv2;
      s2 = s1+((105/4*v3*sf0+735/8*v4*h*sf0)*f0inv4+(15/8*v2*h*sf0+...
          (45/4*h2*sf0-105/4*h*cf0)*v3+(-735/8*h2*cf0+165/4*h3*sf0)*v4)*f0inv3+...
          ((-15/8*h2*cf0+3/4*h3*sf0)*v2+(h4*sf0/4-5/2*h3*cf0)*v3+...
          (-85/8*h4*cf0+13/8*h5*sf0)*v4)*f0inv2+(-h6*v4*cf0/8-h4*v2*cf0/8)*f0inv)*...
          qinv;
      s3 = s2+(-86625/64*f0inv5*v4*sf0+(-693/64*v2*sf0-945/16*v3*h*sf0+...
          (-46305/64*h2*sf0+86625/64*h*cf0)*v4)*f0inv4+((-413/64*h2*sf0+...
          693/64*h*cf0)*v2+(-105/4*h3*sf0+945/16*h2*cf0)*v3+...
          (-4935/64*h4*sf0+8715/32*h3*cf0)*v4)*f0inv3+((91/32*h3*cf0-...
          7/8*h4*sf0)*v2+(-15/16*h5*sf0+105/16*h4*cf0)*v3+(525/32*h5*cf0-...
          245/96*h6*sf0)*v4)*f0inv2+((-h6*sf0/64+161/960*h5*cf0)*v2+...
          h6*v3*cf0/16+(-h8*sf0/64+13/48*h7*cf0)*v4)*f0inv)*qinv2;
      s6 = ((32175/64*v3*sf0+1126125/256*v4*h*sf0)*f0inv5+(7623/256*v2*h*sf0+...
          (-32175/64*h*cf0+40095/128*h2*sf0)*v3+(273735/128*h3*sf0-1126125/...
          256*h2*cf0)*v4)*f0inv4+((945/64*h3*sf0-7623/256*h2*cf0)*v2+(6225/...
          128*h4*sf0-18645/128*h3*cf0)*v3+(39165/256*h5*sf0-172095/256*h4*cf0)*...
          v4)*f0inv3+((147/128*h5*sf0-1239/256*h4*cf0)*v2+(335/192*h6*sf0-...
          45/4*h5*cf0)*v3+(-6825/256*h6*cf0+175/48*h7*sf0)*v4)*f0inv2+...
          ((43/1920*h7*sf0-63/320*h6*cf0)*v2+(-451/2688*h7*cf0+h8*sf0/128)*v3+...
          (-149/384*h8*cf0+23/768*h9*sf0)*v4)*f0inv-h8*v2*cf0/768-...
          v4*h10*cf0/768)*qinv3;
      s9 = -195270075/4096*f0inv6*v4*sf0+(-765765/4096*v2*sf0-...
          482625/256*v3*h*sf0+(-125900775/4096*h2*sf0+195270075/4096*h*cf0)*...
          v4)*f0inv5+((765765/4096*h*cf0-555555/4096*h2*sf0)*v2+(-495495/...
          512*h3*sf0+482625/256*h2*cf0)*v3+(30405375/2048*h3*cf0-...
          21636615/4096*h4*sf0)*v4)*f0inv4+((75075/1024*h3*cf0-...
          7161/256*h4*sf0)*v2+(-11385/128*h5*sf0+173745/512*h4*cf0)*...
          v3+(1426425/1024*h5*cf0-285285/1024*h6*sf0)*v4)*f0inv3;
      s8 = s9+((-6433/4096*h6*sf0+31493/4096*h5*cf0)*v2+(9075/512*h6*cf0-...
          675/256*h7*sf0)*v3+(22165/512*h7*cf0-10925/2048*h8*sf0)*v4)*f0inv2+...
          ((247/1024*h7*cf0-1003/36864*h8*sf0)*v2+(-103/5376*h9*sf0+501/1792*h8*cf0)*...
          v3+(-1537/36864*h10*sf0+6491/12288*h9*cf0)*v4)*f0inv+(-h10*sf0/12288+127/...
          61440*h9*cf0)*v2+v3*h10*cf0/1536+(5/2048*h11*cf0-h12*sf0/12288)*v4;
      v1a = s3+s6+s8*qinv4;

      s1 = -15/2*f0inv3*v3*sf0+(-3*h2*sf0+15/2*h*cf0)*v3*f0inv2+f0inv*v3*h3*cf0/2;
      s2 = s1+((2205/8*f0inv4*sf0+(975/8*h2*sf0-2205/8*h*cf0)*f0inv3+(33/8*h4*sf0-30*h3*cf0)*f0inv2-...
          f0inv*h5*cf0/4)*v4+(15/4*f0inv3*sf0+(3/2*h2*sf0-15/4*h*cf0)*f0inv2-f0inv*h3*cf0/4)*v2+...
          15/2*f0inv3*v3*h*sf0+(27/8*h3*sf0-15/2*h2*cf0)*v3*f0inv2+(h5*sf0/8-7/8*h4*cf0)*v3*f0inv)*qinv;
      s3 = s2+((-17955/32*f0inv4*h*sf0+(17955/32*h2*cf0-4095/16*h3*sf0)*f0inv3+(2205/32*h4*cf0-375/32*h5*sf0)*f0inv2+...
          (-h7*sf0/16+39/32*h6*cf0)*f0inv)*v4+(-189/32*f0inv3*h*sf0+(-81/32*h3*sf0+189/32*h2*cf0)*f0inv2+...
          (-9/160*h5*sf0+9/16*h4*cf0)*f0inv)*v2-7245/64*f0inv4*v3*sf0+(-1785/32*h2*sf0+7245/64*h*cf0)*v3*f0inv3+...
          (-35/8*h4*sf0+1155/64*h3*cf0)*v3*f0inv2+(7/8*h5*cf0-7/48*h6*sf0)*v3*f0inv+h7*v3*cf0/64)*qinv2;
      s6 = ((2027025/256*f0inv5*sf0+(-2027025/256*h*cf0+1119195/256*h2*sf0)*f0inv4+(-3465/2*h3*cf0+...
          132615/256*h4*sf0)*f0inv3+(2485/128*h6*sf0-7455/64*h5*cf0)*f0inv2+(143/768*h8*sf0-149/64*h7*cf0)*f0inv-...
          h9*cf0/128)*v4+(693/16*f0inv4*sf0+(819/32*h2*sf0-693/16*h*cf0)*f0inv3+(7/2*h4*sf0-357/32*h3*cf0)*f0inv2+...
          (47/480*h6*sf0-119/160*h5*cf0)*f0inv-h7*cf0/160)*v2+18315/64*f0inv4*v3*h*sf0+(-18315/64*h2*cf0+...
          34245/256*h3*sf0)*v3*f0inv3+(-9825/256*h4*cf0+1935/256*h5*sf0)*v3*f0inv2+(389/2688*h7*sf0-9/8*h6*cf0)*...
          v3*f0inv+(h9*sf0/768-13/768*h8*cf0)*v3)*qinv3;
      s9 = (-28603575/1024*f0inv5*h*sf0+(28603575/1024*h2*cf0-3528525/256*h3*sf0)*f0inv4+(4579575/1024*h4*cf0-...
          1093785/1024*h5*sf0)*f0inv3+(-14945/512*h7*sf0+202895/1024*h6*cf0)*f0inv2+(-2815/9216*h9*sf0+1745/512*h8*...
          cf0)*f0inv-h11*sf0/1536+59/3072*h10*cf0)*v4+(-135135/1024*f0inv4*h*sf0+(-67221/1024*h3*sf0+135135/1024*h2*...
          cf0)*f0inv3+(693/32*h4*cf0-5355/1024*h5*sf0)*f0inv2+(-63/512*h7*sf0+483/512*h6*cf0)*f0inv+27/2560*h8*...
          cf0-7/15360*h9*sf0)*v2-11550825/4096*f0inv5*v3*sf0+(11550825/4096*h*cf0-3571425/2048*h2*sf0)*v3*...
          f0inv4;
      s8 = s9+(-139425/512*h4*sf0+3292575/4096*h3*cf0)*v3*f0inv3+(2145/32*h5*cf0-6185/512*h6*sf0)*v3*f0inv2+...
          (-15095/86016*h8*sf0+46775/28672*h7*cf0)*v3*f0inv+(-3/2048*h10*sf0+4345/258048*h9*cf0)*v3+...
          1/f0inv*h11*v3*cf0/12288;
      dv1a = s3+s6+s8*qinv4;
      
      %second order correction
      
        v44 = v4*v4;
        v22 = v2*v2;
        v33 = v3*v3;
        v23 = v2*v3;
        v24 = v2*v4;
        v34 = v4*v3;

     s1=(-99225/2*v34*sf0+11025/8*h*sf0*v44)*f0inv6+(-1575/4*v23*sf0+(-11025/8*h2*cf0+5775/8*h3*sf0)*...
         v44+99225/2*v34*cf0*h-22995*v34*h2*sf0-225/8*h*sf0*v33+315/4*v24*h*sf0)*f0inv5;
     sec(1)=s1+(12915/2*v34*h3*cf0+135/4*h3*v24*sf0-705/4*h2*v23*sf0+(-525/2*h4*cf0+555/8*h5*sf0)*...
         v44-315/4*v24*h2*cf0+9/8*h*sf0*v22+1575/4*v23*cf0*h-1200*h4*v34*sf0+(225/8*h2*cf0-105/8*h3*sf0)*v33)*...
         f0inv4+((-25/2*h6*cf0+5/4*h7*sf0)*v44+(-9/8*h2*cf0+3/8*h3*sf0)*v22-12*v34*h6*sf0+150*v34*h5*cf0+...
         (15/4*h4*cf0-3/4*h5*sf0)*v33+3/4*v24*h5*sf0-15/2*v24*h4*cf0+45*v23*h3*cf0-27/4*v23*h4*sf0)*f0inv3+...
         (-v44*h9*sf0/72+v34*h7*cf0/2-v22*h5*sf0/40+(h6*cf0/8-h7*sf0/56)*v33+v23*h5*cf0/2)*f0inv2;
     s1 = 206531325/32*f0inv7*v44*sf0+(904365/16*v24*sf0+809325/32*sf0*v33+1195425/8*h*v34*sf0+...
         (-206531325/32*cf0*h+24306975/8*h2*sf0)*v44)*f0inv6+(-1195425/8*v34*h2*cf0+(188595/16*h2*sf0-...
         809325/32*cf0*h)*v33+51975/2*v24*h2*sf0+(-28384125/32*h3*cf0+709905/4*h4*sf0)*v44-904365/16*v24*cf0*h+...
         5859/32*sf0*v22+561375/8*v34*h3*sf0+15525/16*v23*h*sf0)*f0inv5;
     s2 = s1+((-107415/32*h3*cf0+20565/32*h4*sf0)*v33+(-5859/32*cf0*h+1281/16*h2*sf0)*v22-40725/2*v34*h4*...
          cf0+32265/8*v34*h5*sf0+(-403725/16*h5*cf0+40355/16*h6*sf0)*v44-15525/16*v23*h2*cf0+10185/8*v24*h4*sf0+...
          7137/16*v23*h3*sf0-114345/16*v24*h3*cf0)*f0inv4;
     sec(2)=s2+(225/4*h7*v34*sf0+(-2745/32*h5*cf0+65/8*h6*sf0)*v33+81/8*h6*v24*sf0+(-5215/32*h7*cf0+...
         155/32*h8*sf0)*v44+87/4*h5*v23*sf0+(39/16*h4*sf0-609/32*h3*cf0)*v22-147*v24*h5*cf0-...
         981/8*v23*h4*cf0-2265/4*v34*h6*cf0)*f0inv3+(23/168*v34*h9*sf0-39/16*v23*h6*cf0-5/16*v24*h7*cf0-...
         15/4*v34*h8*cf0+(17/288*h9*cf0+h10*sf0/72)*v44+79/560*v23*h7*sf0+(-67/112*h7*cf0+11/224*h8*sf0)*v33+...
         (-13/80*h5*cf0+h6*sf0/40)*v22)*f0inv2+(-v33*h9*cf0/224-v22*h7*cf0/160-v44*h11*cf0/288)*f0inv;
     s1 = (-583107525/64*v34*sf0-6583060575/256*h*sf0*v44)*f0inv7+((-3124368975/256*h3*sf0+6583060575/256*h2*...
         cf0)*v44-4420845/128*v23*sf0+583107525/64*v34*cf0*h-22683375/256*h*sf0*v33-24999975/128*v24*h*sf0-...
         73069425/16*v34*h2*sf0)*f0inv6+(24999975/128*v24*h2*cf0+97908525/64*v34*h3*cf0-11679525/128*h3*v24*...
         sf0-2242449/128*h2*v23*sf0-6117705/16*h4*v34*sf0+(-192985965/256*h5*sf0+465007725/128*h4*cf0)*v44-...
         136917/256*h*sf0*v22+(-10660815/256*h3*sf0+22683375/256*h2*cf0)*v33+4420845/128*v23*cf0*h)*...
         f0inv5;
     s2 = s1+(418275/16*v24*h4*cf0+2396295/32*v34*h5*cf0-197271/128*v23*h4*sf0+(-51455/4*h7*sf0+...
         14635525/128*h6*cf0)*v44+384417/64*v23*h3*cf0-646875/128*v24*h5*sf0+(-77085/32*h5*sf0+...
         1549845/128*h4*cf0)*v33-374865/32*v34*h6*sf0+(136917/256*h2*cf0-62109/256*h3*sf0)*v22)*...
         f0inv4;
     s3 = s2+((-2875/48*h9*sf0+135355/128*h8*cf0)*v44+(87525/256*h6*cf0-8965/256*h7*sf0)*v33+93855/64*v34*h7*cf0+...
         19617/64*v23*h5*cf0-9295/64*v34*h8*sf0+43515/64*v24*h6*cf0+(-2731/256*h5*sf0+8235/128*h4*cf0)*v22-...
         3027/64*v23*h6*sf0-4015/64*v24*h7*sf0)*f0inv3;
     sec(3)=s3+(-95/896*v24*h9*sf0+(-353/4480*h7*sf0+709/640*h6*cf0)*v22+...
         14851/1344*v34*h9*cf0+(-269/1792*h9*sf0+2327/896*h8*cf0)*v33-...
         6001/13440*v23*h8*sf0+235/64*v24*h8*cf0+(4663/2304*h10*cf0-...
         6899/177408*h11*sf0)*v44-101/168*v34*h10*sf0+771/140*...
         v23*h7*cf0)*f0inv2+(25/1344*v34*h11*cf0+(7/1728*h12*cf0-h13*sf0/2304)*v44+...
         (7/960*h8*cf0-h9*sf0/1280)*v22+11/560*v23*h9*cf0+(-h11*sf0/1792+7/768*h10*cf0)*v33)*f0inv;
     s2 = (2522101725/1024*sf0*v33+10468355625/256*h*v34*sf0+2804997195/512*v24*sf0+(-1126100941875/1024*...
         cf0*h+298602056025/512*h2*sf0)*v44)*f0inv7+((-2522101725/1024*cf0*h+688512825/512*h2*sf0)*v33+...
         (-221837131425/1024*h3*cf0+31483874175/512*h4*sf0)*v44-2804997195/512*v24*cf0*h-...
         10468355625/256*v34*h2*cf0+763416225/256*v24*h2*sf0+5067998325/256*v34*h3*sf0+70212285/...
         512*v23*h*sf0+9401535/1024*sf0*v22)*f0inv6;
     s3 = s2+(87272757/256*v24*h4*sf0+354526965/256*v4*v3*h5*sf0+33859683/512*v23*h3*sf0-591833385/512*v2...
         *v4*h3*cf0-70212285/512*v23*h2*cf0+(621375495/256*h6*sf0-7023251625/512*h5*cf0)*v44-...
         789273225/128*v34*h4*cf0+(-9401535/1024*cf0*h+1334619/256*h2*sf0)*v22+(159234075/1024*h4*sf0-...
         536325075/1024*h3*cf0)*v33)*f0inv5;
     s1 = s3+(862983/64*h6*v24*sf0+1157409/256*h5*v23*sf0-19800495/256*v24*h5*cf0+1040355/32*h7*v34*...
         sf0-15243735/64*v34*h6*cf0+(3238125/512*h6*sf0-36505755/1024*h5*cf0)*v33+(-349765845/1024*h7*cf0+...
         9755925/256*h8*sf0)*v44-2613897/128*v23*h4*cf0+(170181/256*h4*sf0-2204631/1024*h3*cf0)*v22)*f0inv4+...
         (41845/128*v34*h9*sf0+91797/512*v24*h8*sf0-230355/64*v34*h8*cf0+51687/512*v23*h7*sf0-...
         389895/512*v23*h6*cf0+(-7675355/2304*h9*cf0+31985/144*h10*sf0)*v44+(-77385/512*h5*cf0+12735/512*h6*sf0)*v22+...
         (327735/3584*h8*sf0-3108765/3584*h7*cf0)*v33-918885/512*v24*h7*cf0)*f0inv3;
     sec(4)=s1+(47807/53760*v23*h9*sf0+16487/26880*v24*h10*sf0-231587/17920*v24*h9*cf0+86117/59136*v34*h11*sf0+...
         (-11255/1536*h9*cf0+1593/3584*h10*sf0)*v33+(-104103/35840*h7*cf0+25589/107520*h8*sf0)*v22-...
         47871/4480*v23*h8*cf0-130927/5376*v34*h10*cf0+(-1902643/177408*h11*cf0+185153/532224*h12*sf0)*v44)*f0inv2+...
         1126100941875/1024*sf0*v44*f0inv8+((-4847/532224*h13*cf0+h14*sf0/1728)*v44-53/3584*v24*h11*cf0-...
         523/8064*v34*h12*cf0+3/1792*v34*h13*sf0-2887/53760*v23*h10*cf0+97/53760*v23*h11*sf0+(-67/3072*h11*cf0+...
         23/21504*h12*sf0)*v33+(h10*sf0/960-1621/107520*h9*cf0)*v22)*f0inv-h15*cf0*v44/27648-v22*h11*cf0/15360-...
         h13*v33*cf0/21504;


      sew(1)=-2767275/8*f0inv6*v44*sf0+(-15975/8*sf0*v33+(-322875/2*h2*sf0+2767275/8*cf0*h)*v44-...
          17325/4*v24*sf0)*f0inv5+(-171/8*sf0*v22-7815/4*v24*h2*sf0+(369075/8*h3*cf0-71025/8*h4*sf0)*v44+...
          (-1845/2*h2*sf0+15975/8*cf0*h)*v33+17325/4*v24*cf0*h)*f0inv4+((9495/8*h5*cf0-435/4*h6*sf0)*v44+...
          (-93/2*h4*sf0+2055/8*h3*cf0)*v33+(171/8*cf0*h-9*h2*sf0)*v22+510*v24*h3*cf0-81*v24*h4*sf0)*f0inv3+...
          ((-h8*sf0/8+25/4*h7*cf0)*v44+(-h6*sf0/4+21/4*h5*cf0)*v33+(-h4*sf0/8+15/8*h3*cf0)*v22-...
          h6*v24*sf0/4+29/4*v24*h5*cf0)*f0inv2+(-v44*h9*cf0/72-v33*h7*cf0/56-v22*h5*cf0/40)*f0inv;
      s1 = (30440025/32*h*sf0*v44+6288975/8*v34*sf0)*f0inv6+(2943405/8*v34*h2*sf0+...
          (14390775/32*h3*sf0-30440025/32*h2*cf0)*v44+66555/16*v23*sf0+143775/32*h*sf0*v33+...
          155925/16*v24*h*sf0-6288975/8*v34*cf0*h)*f0inv5+(7593/4*h2*v23*sf0+165105/8*h4*v34*sf0+...
          1197/32*h*sf0*v22-105885*v34*h3*cf0+(-143775/32*h2*cf0+67635/32*h3*sf0)*v33+...
          (-1061025/8*h4*cf0+433125/16*h5*sf0)*v44+36015/8*h3*v24*sf0-155925/16*v24*h2*cf0-...
          66555/16*v23*cf0*h)*f0inv4;
      s2 = s1+(87*v23*h4*sf0+(531/32*h3*sf0-1197/32*h2*cf0)*v22+(-9855/16*h4*cf0+3915/32*h5*sf0)*v33+...
          915/4*v24*h5*sf0+(13845/32*h7*sf0-127995/32*h6*cf0)*v44-8187/16*v23*h3*cf0-...
          20055/16*v24*h4*cf0-5625/2*v34*h5*cf0+1055/4*v34*h6*sf0)*f0inv3;
      sew(2)=s2+(3/8*v23*h6*sf0-55/2*v24*h6*cf0+(9/16*h5*sf0-33/8*h4*cf0)*v22+3/8*v34*h8*sf0+...
          (53/32*h9*sf0-535/16*h8*cf0)*v44+2*v24*h7*sf0-31/2*v4*v3*h7*cf0+(-135/8*h6*cf0+3/2*h7*sf0)*v33-...
          71/8*v23*h5*cf0)*f0inv2+((-h9*sf0/224-3/56*h8*cf0)*v33+(-3/160*h6*cf0-h7*sf0/160)*v22-...
          v24*h8*cf0/16+(-h11*sf0/288-7/288*h10*cf0)*v44+9/560*v23*h7*cf0+v34*h9*cf0/84)*f0inv;
      s2= -22910071275/256*f0inv7*v44*sf0+((22910071275/256*cf0*h-5630154075/128*h2*sf0)*v44-...
          81756675/32*h*v34*sf0-66891825/256*sf0*v33-74189115/128*v24*sf0)*f0inv6;
      s3 = s2+(-336501/256*sf0*v22-19333215/16*v34*h3*sf0-36384975/128*v24*h2*sf0+81756675/32*v34*h2*cf0+...
          (3623617725/256*h3*cf0-856622295/256*h4*sf0)*v44-732105/64*v23*h*sf0+(66891825/256*cf0*h-...
          2059695/16*h2*sf0)*v33+74189115/128*v24*cf0*h)*f0inv5;
      s1 = s3+((-82161/128*h2*sf0+336501/256*cf0*h)*v22-339903/64*v23*h3*sf0+(10657845/256*h3*cf0-...
          1269765/128*h4*sf0)*v33-682137/32*v24*h4*sf0+(-11712465/128*h6*sf0+157862415/256*h5*cf0)*v44+...
          5827635/64*v24*h3*cf0-2333025/32*v34*h5*sf0+11414205/32*v34*h4*cf0+732105/64*v23*h2*cf0)*f0inv4+...
          (-18635/16*h7*v34*sf0+(355255/32*h7*cf0-141095/128*h8*sf0)*v44+(59175/32*h5*cf0-17805/64*h6*sf0)*v33+...
          (52155/256*h3*cf0-11969/256*h4*sf0)*v22+23967/16*v23*h4*cf0-69775/128*h6*v24*sf0-17931/64*h5*v23*sf0+...
          492105/128*v24*h5*cf0+345105/32*v34*h6*cf0)*f0inv3;
     sew(3)=s1+(-43/16*v23*h7*sf0+(8691/256*h7*cf0-831/256*h8*sf0)*v33-139/32*v34*h9*sf0-...
         1931/384*v24*h8*sf0+1933/32*v24*h7*cf0+561/16*v23*h6*cf0+1427/16*v34*h8*cf0+...
         (-163/160*h6*sf0+10309/1280*h5*cf0)*v22+(-6221/1152*h10*sf0+101507/1152*h9*cf0)*v44)*f0inv2+...
         (3797/13440*v24*h9*cf0+v34*h11*sf0/336+(12895/59136*h11*cf0-11/6912*h12*sf0)*v44+9/2240*v23*...
         h9*sf0-v24*h10*sf0/128+(1069/13440*h7*cf0+h8*sf0/3840)*v22+87/1120*v23*h8*cf0+55/672*v34*h10*cf0+...
         (383/1792*h9*cf0-13/2688*h10*sf0)*v33)*f0inv-h11*v33*cf0/1792-h13*v44*cf0/2304-h9*v22*cf0/1280;
      s2=(389471211675/1024*h*sf0*v44+13154102325/128*v4*v3*sf0)*f0inv7+(1003377375/1024*h*sf0*v33+...
          (187857490275/1024*h3*sf0-389471211675/1024*h2*cf0)*v44+13728004875/256*v34*h2*sf0+...
          1112836725/512*v24*h*sf0+151872435/512*v23*sf0-13154102325/128*v34*cf0*h)*f0inv6+(-151872435/512*v23*cf0*h+...
          40402791/256*h2*v23*sf0+(-1003377375/1024*h2*cf0+481601835/1024*h3*sf0)*v33+133082235/128*h3*v24*sf0+...
          (6430079955/512*h5*sf0-29016876525/512*h4*cf0)*v44+4374513/1024*h*sf0*v22-4958603325/256*v34*h3*cf0+...
          1363379355/256*h4*v34*sf0-1112836725/512*v24*h2*cf0)*f0inv5;
      s3 = s2+(17314209/256*v24*h5*sf0+(-4374513/1024*h2*cf0+2065041/1024*h3*sf0)*v22+(-73571355/512*h4*cf0+...
          32020065/1024*h5*sf0)*v33+2138895/128*v23*h4*sf0+(289352895/1024*h7*sf0-2170491375/1024*h6*cf0)*v44+...
          12710325/64*v34*h6*sf0-30181437/512*v23*h3*cf0-161383365/512*v24*h4*cf0-147569175/128*v34*h5*cf0)*...
          f0inv4;
      s1 = s3+((-2634885/512*h6*cf0+85185/128*h7*sf0)*v33+39213/64*v23*h6*sf0-6974295/256*v34*h7*cf0+756075/256*v34*h8*...
          sf0-1390917/128*v24*h6*cf0-467511/128*v23*h5*cf0+(1386075/512*h9*sf0-15608515/512*h8*cf0)*v44+...
          (61605/512*h5*sf0-303435/512*h4*cf0)*v22+345525/256*v24*h7*sf0)*f0inv3+((3049/512*h9*sf0-35589/512*h8*cf0)*v33+...
          15713/1536*v24*h9*sf0+(-90657/5120*h6*cf0+9847/5120*h7*sf0)*v22-1387101/17920*v23*h7*cf0-67581/512*v24*h8*cf0+...
          (1757/144*h11*sf0-920821/4608*h10*cf0)*v44+377033/53760*v23*h8*sf0-110977/448*v34*h9*cf0+...
          27387/1792*v34*h10*sf0)*f0inv2;
      sew(4)=s1+(9/1280*v23*h10*sf0-4265/10752*v23*h9*cf0+121/4480*v2*h11*v4*sf0+(-2939/7168*h10*cf0+421/21504*...
          h11*sf0)*v33+(-671/4480*h8*cf0+45/7168*h9*sf0)*v22+19/2304*h12*v34*sf0-33247/53760*h10*v24*cf0+...
          (2465/133056*h13*sf0-23213/39424*h12*cf0)*v44-35851/59136*v34*h11*cf0)*f0inv+(-h12*cf0/5376-h13*sf0/21504)*v33+...
          9/17920*h11*v23*cf0+h13*v34*cf0/2688+(-h15*sf0/27648+h14*cf0/27648)*v44+(-h11*sf0/15360+h10*cf0/3072)*v22-...
          h12*v24*cf0/1536;

      ses(1)=2789325/8*f0inv7*v44*sf0+(15525/8*sf0*v33+(163800*h2*sf0-2789325/8*cf0*h)*v44+17955/4*v24*sf0)*f0inv6+...
          ((-15525/8*cf0*h+1755/2*h2*sf0)*v33-17955/4*v24*cf0*h+(-380625/8*h3*cf0+75495/8*h4*sf0)*v44+...
          8295/4*v24*h2*sf0+189/8*sf0*v22)*f0inv5+((75/2*h4*sf0-1845/8*h3*cf0)*v33+(-10605/8*h5*cf0+...
          525/4*h6*sf0)*v44-1155/2*v24*h3*cf0+105*v24*h4*sf0+(21/2*h2*sf0-189/8*cf0*h)*v22)*f0inv4+(3/4*h6*v24*sf0+...
          (h6*sf0/4-15/4*h5*cf0)*v33+(-35/4*h7*cf0+3/8*h8*sf0)*v44+(-21/8*h3*cf0+3/8*h4*sf0)*v22-...
          49/4*v24*h5*cf0)*f0inv3+(-v44*h9*cf0/72-v33*h7*cf0/56-v22*h5*cf0/40)*f0inv2;

      s1 = (-36261225/32*h*sf0*v44-6081075/8*v34*sf0)*f0inv7+(6081075/8*v34*cf0*h-63855/16*v23*sf0-...
          2843775/8*v34*h2*sf0-197505/16*v24*h*sf0-170775/32*h*sf0*v33+(-17203725/32*h3*sf0+36261225/32*h2*cf0)*...
          v44)*f0inv6+(-7317/4*h2*v23*sf0-46305/8*h3*v24*sf0-158715/8*h4*v34*sf0+(170775/32*h2*cf0-78705/32*h3*sf0)*v33+...
          197505/16*v24*h2*cf0+63855/16*v23*cf0*h-1701/32*h*sf0*v22+408375/4*v34*h3*cf0+(2558325/16*h4*cf0-...
          66255/2*h5*sf0)*v44)*f0inv5;
      s2 = s1+(-1025/4*v34*h6*sf0-2625/8*v24*h5*sf0+(1701/32*h2*cf0-777/32*h3*sf0)*v22+7983/16*v23*h3*cf0+...
          2700*v34*h5*cf0+(-17955/32*h7*sf0+160335/32*h6*cf0)*v44+(5445/8*h4*cf0-3945/32*h5*sf0)*v33+26775/16*...
          v24*h4*cf0-699/8*v23*h4*sf0)*f0inv4;
     ses(2)=s2+(-4*v24*h7*sf0+39/4*v23*h5*cf0+65/4*v4*v3*h7*cf0+357/8*v24*h6*cf0-5/8*v34*h8*sf0+(105/16*h4*cf0-...
         9/8*h5*sf0)*v22+(15*h6*cf0-5/4*h7*sf0)*v33+(735/16*h8*cf0-85/32*h9*sf0)*v44-5/8*v23*h6*sf0)*f0inv3+...
         (v34*h9*cf0/84+9/560*v23*h7*cf0+3/16*v24*h8*cf0+(-h9*sf0/224+9/112*h8*cf0)*v33+(31/288*h10*cf0-...
         h11*sf0/288)*v44+(19/160*h6*cf0-h7*sf0/160)*v22)*f0inv2;
      s2=(91216125/32*h*v34*sf0+(-22135829625/256*cf0*h+5531096025/128*h2*sf0)*v44+63609975/256*sf0*v33+...
          71216145/128*v24*sf0)*f0inv7+(36068175/128*v24*h2*sf0-91216125/32*v34*h2*cf0+...
          (8011575/64*h2*sf0-63609975/256*cf0*h)*v33+21525075/16*v34*h3*sf0-71216145/128*v24*cf0*h+...
          320463/256*sf0*v22+(915059925/256*h4*sf0-3683582175/256*h3*cf0)*v44+830115/64*v23*h*sf0)*f0inv6;
      s1 = s2+((1366695/128*h4*sf0-10842975/256*h3*cf0)*v33-3082365/32*v24*h3*cf0+(83655/128*h2*sf0-...
          320463/256*cf0*h)*v22+791307/32*v24*h4*sf0+(-179106525/256*h5*cf0+14180955/128*h6*sf0)*v44-...
          830115/64*v23*h2*cf0-12644775/32*v34*h4*cf0+385803/64*v23*h3*sf0+2562615/32*v34*h5*sf0)*f0inv5+...
          ((-133155/64*h5*cf0+20145/64*h6*sf0)*v33-637989/128*v24*h5*cf0+(-57325/4*h7*cf0+193295/128*h8*sf0)*v44+...
          20025/16*h7*v34*sf0+102369/128*h6*v24*sf0+(-60489/256*h3*cf0+16479/256*h4*sf0)*v22-...
          374715/32*v34*h6*cf0-54549/32*v23*h4*cf0+20565/64*h5*v23*sf0)*f0inv4;
      ses(3)=s1+(-1323/32*v23*h6*cf0-1545/16*v34*h8*cf0+165/32*v34*h9*sf0+3757/384*v24*h8*sf0-3243/32*v24*h7*cf0+...
          (339/160*h6*sf0-17187/1280*h5*cf0)*v22+(5815/1792*h8*sf0-65595/1792*h7*cf0)*v33+(9715/1152*h10*sf0-...
          147115/1152*h9*cf0)*v44+113/32*v23*h7*sf0)*f0inv3+((-74659/177408*h11*cf0+109/6912*h12*sf0)*v44...
          -39/224*v34*h10*cf0+(5/384*h10*sf0-1207/5376*h9*cf0)*v33+3/128*v24*h10*sf0+v34*h11*sf0/336-...
          2939/4480*v24*h9*cf0-101/560*v23*h8*cf0+(73/3840*h8*sf0-219/896*h7*cf0)*v22+9/2240*v23*h9*sf0)*...
          f0inv2+(-h11*v33*cf0/1792-h9*v22*cf0/1280-h13*v44*cf0/2304)*f0inv+22135829625/256*sf0*v44*f0inv8;
      s2 = (-13544940825/256*v34*h2*sf0+(420580762875/1024*h2*cf0-204028686225/1024*h3*sf0)*v44-144073215/512*...
          v23*sf0-1081369575/1024*h*sf0*v33-1210674465/512*v24*h*sf0+12598574625/128*v34*cf0*h)*f0inv7+...
          (-36660195/32*h3*v24*sf0+144073215/512*v23*cf0*h-40212315/256*h2*v23*sf0+5145891075/256*v34*h3*cf0+...
          1210674465/512*v24*h2*cf0+(-522103725/1024*h3*sf0+1081369575/1024*h2*cf0)*v33-...
          4806945/1024*h*sf0*v22-1492782525/256*h4*v34*sf0+(7979387325/128*h4*cf0-3611943075/256*h5*sf0)*v44)*...
          f0inv6;
      s1 = s2+((2515645275/1024*h6*cf0-350003745/1024*h7*sf0)*v44+(4806945/1024*h2*cf0-2333331/1024*h3*sf0)*v22+...
          183004965/512*v24*h4*cf0+32400225/512*v23*h3*cf0+168711075/128*v34*h5*cf0+(5051475/32*h4*cf0-...
          35913735/1024*h5*sf0)*v33-7467135/32*v34*h6*sf0-4940793/256*v23*h4*sf0-1288287/16*v24*h5*sf0)*f0inv5+...
          ((-20691/128*h5*sf0+91377/128*h4*cf0)*v22+(-100155/128*h7*sf0+3030885/512*h6*cf0)*v33+...
          (19947525/512*h8*cf0-1892425/512*h9*sf0)*v44+1141569/256*v23*h5*cf0+3563703/256*v24*h6*cf0+...
          8288865/256*v34*h7*cf0-490479/256*v24*h7*sf0-897705/256*v34*h8*sf0-99429/128*v23*h6*sf0)*f0inv4;
      s2 = s1+(109659/512*v24*h8*cf0+362307/3584*v23*h7*cf0+(147345/1792*h8*cf0-3105/448*h9*sf0)*v33+...
          (28677/1024*h6*cf0-3909/1024*h7*sf0)*v22+98405/336*v4*v3*h9*cf0+(1353565/4608*h10*cf0-...
          44825/2304*h11*sf0)*v44-9963/512*v24*h9*sf0-98915/5376*v34*h10*sf0-34455/3584*v23*h8*sf0)*f0inv3;
      ses(4)=s2+(74771/53760*h10*v24*cf0-47/1792*v23*h10*sf0+49069/59136*v34*h11*cf0-953/13440*v2*h11*v4*...
          sf0+(10253/21504*h10*cf0-599/21504*h11*sf0)*v33+(-3679/107520*h9*sf0+3687/8960*h8*cf0)*v22+...
          34189/53760*v23*h9*cf0-395/16128*h12*v34*sf0+(93601/88704*h12*cf0-48733/1064448*h13*sf0)*v44)*...
          f0inv2+(-12598574625/128*v34*sf0-420580762875/1024*h*sf0*v44)*f0inv8+(h13*v34*cf0/2688+h12*v24*cf0/512+...
          (5/3584*h12*cf0-h13*sf0/21504)*v33+(-h15*sf0/27648+43/27648*h14*cf0)*v44+9/17920*h11*v23*cf0+...
          (-h11*sf0/15360+31/15360*h10*cf0)*v22)*f0inv;
      
      s1 = (99225/2*v34*sf0+11025/8*h*sf0*v44)*f0inv6+(-225/8*h*sf0*v33-99225/2*v34*cf0*h+(-11025/8*h2*...
          cf0+5775/8*h3*sf0)*v44+1575/4*v23*sf0+22995*v34*h2*sf0+315/4*v24*h*sf0)*f0inv5;
      sez(1)=s1+(-12915/2*v34*h3*cf0+9/8*h*sf0*v22+(-525/2*h4*cf0+555/8*h5*sf0)*v44-...
          315/4*v24*h2*cf0+705/4*h2*v23*sf0+135/4*h3*v24*sf0+(225/8*h2*cf0-105/8*h3*sf0)*v33+...
          1200*h4*v34*sf0-1575/4*v23*cf0*h)*f0inv4+(3/4*v24*h5*sf0+(15/4*h4*cf0-3/4*h5*sf0)*v33-...
          150*v34*h5*cf0-15/2*v24*h4*cf0+(-25/2*h6*cf0+5/4*h7*sf0)*v44+(-9/8*h2*cf0+3/8*h3*sf0)*v22-...
          45*v23*h3*cf0+12*v34*h6*sf0+27/4*v23*h4*sf0)*f0inv3+(-v44*h9*sf0/72+(h6*cf0/8-h7*sf0/56)*...
          v33-v34*h7*cf0/2-v23*h5*cf0/2-v22*h5*sf0/40)*f0inv2;
      s1 = -206531325/32*f0inv7*v44*sf0+(-904365/16*v24*sf0-809325/32*sf0*v33-987525/8*h*v34*sf0+...
          (206531325/32*cf0*h-48735225/16*h2*sf0)*v44)*f0inv6+(987525/8*v34*h2*cf0+904365/16*v24*cf0*h-...
          12825/16*v23*h*sf0-464895/8*v34*h3*sf0+(809325/32*cf0*h-93285/8*h2*sf0)*v33-5859/32*sf0*v22-...
          210735/8*v24*h2*sf0+(28626675/32*h3*cf0-725655/4*h4*sf0)*v44)*f0inv5;
      s2 = s1+((106575/4*h5*cf0-11585/4*h6*sf0)*v44+16965*v4*v3*h4*cf0+120015/16*v24*h3*cf0+12825/16*v23*h2*...
          cf0-5883/16*v23*h3*sf0-27165/8*v34*h5*sf0-11445/8*v24*h4*sf0+(-18645/32*h4*sf0+103365/32*h3*cf0)*v33+...
          (-84*h2*sf0+5859/32*cf0*h)*v22)*f0inv4;     
      sez(2)=s2+((7525/32*h7*cf0-455/32*h8*sf0)*v44-195/4*h7*v34*sf0+(2175/32*h5*cf0-35/8*h6*sf0)*v33+...
          1491/8*v24*h5*cf0+(-63/16*h4*sf0+735/32*h3*cf0)*v22+1935/4*v34*h6*cf0+201/2*v23*h4*cf0-...
          63/4*h6*v24*sf0-141/8*h5*v23*sf0)*f0inv3+(31/16*v23*h6*cf0+11/16*v24*h7*cf0+(163/288*h9*cf0+...
          h10*sf0/144)*v44+(-3/112*h7*cf0+9/224*h8*sf0)*v33-61/560*v23*h7*sf0-19/168*v34*h9*sf0+...
          (7/20*h5*cf0+h6*sf0/80)*v22+13/4*v34*h8*cf0)*f0inv2+(-v33*h9*cf0/224-v22*h7*cf0/160-v44*h11*cf0/288)*f0inv;
      s1 = (5808818925/256*h*sf0*v44+583107525/64*v34*sf0)*f0inv7+(22027005/128*v24*h*sf0-583107525/64*v34*...
          cf0*h+71718075/16*v34*h2*sf0+19401525/256*h*sf0*v33+4420845/128*v23*sf0+(-5808818925/256*h2*cf0+...
          2763913425/256*h3*sf0)*v44)*f0inv6+(10357875/128*h3*v24*sf0-22027005/128*v24*h2*cf0+2183049/128*h2*...
          v23*sf0-92503125/64*v34*h3*cf0+(9083745/256*h3*sf0-19401525/256*h2*cf0)*v33-...
          4420845/128*v23*cf0*h+(173694195/256*h5*sf0-413820225/128*h4*cf0)*v44+5483115/16*h4*v34*sf0+...
          120879/256*h*sf0*v22)*f0inv5;
      s2 = s1+(169299/128*v23*h4*sf0-354717/64*v23*h3*cf0+600525/128*v24*h5*sf0+(-120879/256*h2*cf0+55827/256*h3*sf0)*...
          v22+(127665/64*h5*sf0-1308285/128*h4*cf0)*v33-2028015/32*v34*h5*cf0-753885/32*v24*...
          h4*cf0+301785/32*v34*h6*sf0+(98375/8*h7*sf0-13449455/128*h6*cf0)*v44)*f0inv4;
      s3 = s2+(-42417/64*v24*h6*cf0+(2813/256*h5*sf0-7767/128*h4*cf0)*v22+4217/64*v24*h7*sf0-...
          73095/64*v34*h7*cf0+2211/64*v23*h6*sf0-15531/64*v23*h5*cf0+(14015/192*h9*sf0-...
          139955/128*h8*cf0)*v44+(6475/256*h7*sf0-69615/256*h6*cf0)*v33+7135/64*v34*h8*sf0)*f0inv3;
      sez(3)=s3+(-11323/1344*v34*h9*cf0+(h9*sf0/256-1241/896*h8*cf0)*v33-275/64*v24*h8*cf0-...
          4173/1120*v23*h7*cf0+3781/13440*v23*h8*sf0+(-8069/2304*h10*cf0+...
          17125/177408*h11*sf0)*v44+129/896*v24*h9*sf0+(-803/640*h6*cf0+291/4480*h7*sf0)*v22+...
          25/56*v34*h10*sf0)*f0inv2+(-17/1344*v34*h11*cf0+(-h11*sf0/1792+37/5376*h10*cf0)*v33+...
          (-h9*sf0/1280+h8*cf0/240)*v22-13/1120*v23*h9*cf0+(h12*cf0/432-h13*sf0/2304)*v44)*f0inv;
      s2 = (-2522101725/1024*sf0*v33-9357300225/256*h*v34*sf0+(1126100941875/1024*cf0*h-...
          18251312625/32*h2*sf0)*v44-2804997195/512*v24*sf0)*f0inv7+(-62413065/512*v23*h*sf0-...
          9401535/1024*sf0*v22+9357300225/256*v34*h2*cf0-4501683225/256*v34*h3*sf0-370559475/128*...
          v24*h2*sf0+(2522101725/1024*cf0*h-331949475/256*h2*sf0)*v33+2804997195/512*v24*cf0*h+...
          (-28377570975/512*h4*sf0+208675023375/1024*h3*cf0)*v44)*f0inv6;
      s3 = s2+((-2564991/512*h2*sf0+9401535/1024*cf0*h)*v22+(-1058058885/512*h6*sf0+1527658275/128*h5*cf0)*v44+...
          691291575/128*v34*h4*cf0+(487097325/1024*h3*cf0-135836415/1024*h4*sf0)*v33+62413065/512*v23*h2*cf0-...
          303451785/256*v34*h5*sf0-76925277/256*v24*h4*sf0+547238835/512*v24*h3*cf0-29810781/512*v23*h3*sf0)*f0inv5;
      s1 = s3+((29517345/1024*h5*cf0-2506515/512*h6*sf0)*v33+(-146817/256*h4*sf0+1996137/1024*h3*cf0)*v22-...
          828105/32*h7*v34*sf0-240075/64*h5*v23*sf0+4503213/256*v23*h4*cf0+12632685/64*v34*h6*cf0+...
          263835/4*v24*h5*cf0-2906937/256*h6*v24*sf0+(-524205/16*h8*sf0+296302695/1024*h7*cf0)*v44)*...
          f0inv4+(-80637/512*v24*h8*sf0+(2311815/3584*h7*cf0-231345/3584*h8*sf0)*v33+43845/16*v34*h8*cf0+...
          (32703/256*h5*cf0-1359/64*h6*sf0)*v22-30515/128*v34*h9*sf0+305415/512*v23*h6*cf0-...
          37293/512*v23*h7*sf0+(6869765/2304*h9*cf0-495155/2304*h10*sf0)*v44+781275/512*v24*h7*cf0)*f0inv3;
      sez(4)=s1+((24943/5376*h9*cf0-355/1792*h10*sf0)*v33+(91041/35840*h7*cf0-21053/107520*h8*sf0)*v22+...
          215207/17920*v24*h9*cf0-57961/59136*v34*h11*sf0-1037/1680*v24*h10*sf0+7743/1120*v23*h8*cf0+...
          (2110697/177408*h11*cf0-249581/532224*h12*sf0)*v44-9053/17920*v23*h9*sf0+30621/1792*v34*h10*cf0)*...
          f0inv2-1126100941875/1024*sf0*v44*f0inv8+(47/1152*v34*h12*cf0-43/53760*v23*h11*sf0+...
          (h10*sf0/1536+577/107520*h9*cf0)*v22-5/5376*v34*h13*sf0+(5/13824*h14*sf0+9787/1064448*h13*cf0)*v44+...
          59/3584*v24*h11*cf0+(17/21504*h12*sf0-17/21504*h11*cf0)*v33+41/1536*v23*h10*cf0)*f0inv-...
          h15*cf0*v44/27648-v22*h11*cf0/15360-h13*v33*cf0/21504;
      
	qpow=qinv.^(0:3);
    qpow=qpow';
    u2a=sec*qpow; 
 	du2a=sew*qpow;
	v2a=ses*qpow;
	dv2a=sez*qpow;

        
	pu=u0a+u1a+u2a;
    pv=v0a+v1a+v2a;
	pup=du0a+du1a+du2a;
    pvp=dv0a+dv1a+dv2a;
    
%     u0e=-1/2*sf0*h/f0e+1/2*(h2*f0e*sf0+sf0-f0e*h*cf0)*f1/f0e3+...
%         1/4*(-h2*sf0-1/2*h3*f0e*cf0-1/2*sf0/f0e+1/2*cf0*h)*f1/f0e2+...
%         5/192*h*(-21*sf0/f0e+21*h*cf0-10*h2*sf0+...
%         3*h3*f0e*cf0)*f12/f0e3+1/96*h*(-1/2*h2*sf0/f0e+1/2*h3*cf0-...
%         3/2*h4*sf0)*f12/f0e2+5/384*(h6*f0e3*sf0+105*sf0+...
%         105*h2*f0e*sf0-7*h5*f0e3*cf0-105*f0e*h*cf0+28*h4*f0e2*sf0-70*h3*f0e2*cf0)*f13/f0e6+...
%         1/384*(-1/2*h7*f0e3*cf0-105/2*sf0/f0e+105/2*cf0*h-105*h2*sf0+175/2*h3*f0e*cf0+7*h5*f0e2*cf0-...
%         35*h4*f0e*sf0)*f13/f0e5+11/36864*h*(238*h5*f0e2*cf0-36*h6*f0e2*sf0+3*h7*f0e3*cf0-...
%         7350*h2*sf0-12285*sf0/f0e+3255*h3*f0e*cf0+12285*h*cf0-...
%         1050*h4*f0e*sf0)*f14/f0e6+1/18432*h*(-70*h5*f0e*cf0-11*h6*f0e*sf0+...
%         15/2*h7*f0e2*cf0-3/2*h^8*f0e2*sf0+2415/2*h2*sf0/f0e-2415/2*h3*cf0+...
%         945/2*h4*sf0)*f14/f0e5;
%     
%     
%     v0e=-1/2*cf0*h/f0e+1/2*sf0/f0e2+3/8*h*(h*cf0-sf0/f0e)*f1/f0e2-...
%         1/8*h3*sf0*f1/f0e2+1/24*(-15*f0e*h*cf0+3*h4*f0e2*sf0-10*h3*f0e2*cf0+15*sf0+15*h2*f0e*sf0)*f12/f0e5+...
%         1/96*(15/2*cf0*h-15*h2*sf0-5/2*h4*f0e*sf0-3/2*h5*f0e2*cf0+25/2*h3*f0e*cf0-15/2*sf0/f0e)*f12/f0e4+...
%         5/384*h*(-60*h2*f0e*sf0+h5*f0e3*cf0-7*h4*f0e2*sf0+25*h3*f0e2*cf0+105*f0e*h*cf0-105*sf0)*f13/f0e6+...
%         1/384*h*(75/2*h2*sf0-20*h3*f0e*cf0+1/2*h5*f0e2*cf0-1/2*h6*f0e2*sf0+5*h4*f0e*sf0-105/2*cf0*h+...
%         105/2*sf0/f0e)*f13/f0e5+13/36864*(10395*h2*sf0-10395*h*cf0+3045*h4*f0e*sf0+...
%         10395*sf0/f0e-6930*h3*f0e*cf0-966*h5*f0e2*cf0+226*h6*f0e2*sf0+3*h^8*f0e3*sf0-...
%         36*h7*f0e3*cf0)*f14/f0e^(7)+1/18432*(-10395/2*h2*sf0/f0e+10395/2*h3*cf0-...
%         2625*h4*sf0+1785/2*h5*f0e*cf0-195*h6*f0e*sf0+13*h7*f0e2*cf0+...
%         6*h^8*f0e2*sf0-3/2*h^9*f0e3*cf0)*f14/f0e6;
%     
%     u0pe=-1/2*sf0/f0e-1/2*cf0*h+1/4*h*(-cf0*h-1/2*h2*sf0)*f1/f0e+...
%         1/8*h*(h*cf0+sf0/f0e)*f1/f0e+1/32*(-21*sf0+2*h3*f0e2*cf0-...
%         9*h2*f0e*sf0+21*f0e*h*cf0+3*h4*f0e2*sf0)*f12/f0e4+1/96*(21/2*sf0/f0e-21/2*cf0*h+...
%         1/2*h3*f0e*cf0-17/2*h4*f0e*sf0+3*h2*sf0-3/2*h5*f0e2*cf0)*f12/f0e3+7/768*h*(-7*h3*f0e*cf0+...
%         42*h2*sf0-105*h*cf0+h5*f0e2*cf0+105*sf0/f0e-h4*f0e*sf0)*f13/f0e4+1/384*h*(-21/2*h3*cf0+11/2*h4*sf0+...
%         21/2*h2*sf0/f0e-2*h5*f0e*cf0-1/2*h6*f0e*sf0)*f13/f0e3+...
%         1/3072*(378*h5*f0e3*cf0-14*h6*f0e3*sf0+5670*h3*f0e2*cf0-9765*h2*f0e*sf0-...
%         12285*sf0-1995*h4*f0e2*sf0-12*h7*f0e4*cf0+3*h8*f0e4*sf0+12285*f0e*h*cf0)*f14/f0e7+...
%         1/18432*(-273/2*h5*f0e2*cf0-140*h6*f0e2*sf0+55*h7*f0e3*cf0-12915/2*h3*f0e*cf0+4305/2*h4*f0e*sf0+...
%         8505*h2*sf0+12285/2*sf0/f0e-12285/2*cf0*h-15/2*h8*f0e3*sf0-3/2*h9*f0e4*cf0)*f14/f0e6;
%     
%     v0pe=-1/2*sf0*h/f0e+1/2*(h2*f0e*sf0+f0e*h*cf0-sf0)*f1/f0e3+1/4*(-2*h2*sf0-1/2*h3*f0e*cf0-...
%         1/2*cf0*h+1/2*sf0/f0e)*f1/f0e2+5/192*h*(3*h3*f0e*cf0+2*h2*sf0-...
%         15*h*cf0+15*sf0/f0e)*f12/f0e3+1/96*h*(-11/2*h3*cf0-...
%         3/2*h4*sf0+11/2*h2*sf0/f0e)*f12/f0e2+5/384*(-105*sf0+h6*f0e3*sf0+...
%         40*h3*f0e2*cf0-10*h4*f0e2*sf0-h5*f0e3*cf0-75*h2*f0e*sf0+105*f0e*h*cf0)*f13/f0e6+...
%         1/384*(105/2*sf0/f0e-105/2*cf0*h-3*h6*f0e2*sf0-1/2*h7*f0e3*cf0-85/2*h3*f0e*cf0+5*h4*f0e*sf0+...
%         8*h5*f0e2*cf0+60*h2*sf0)*f13/f0e5+11/36864*h*(3*h7*f0e3*cf0+10395*sf0/f0e-...
%         26*h5*f0e2*cf0-12*h6*f0e2*sf0-1785*h3*f0e*cf0+390*h4*f0e*sf0-...
%         10395*h*cf0+5250*h2*sf0)*f14/f0e6+1/18432*h*(-9/2*h7*f0e2*cf0-...
%         3/2*h^8*f0e2*sf0-130*h5*f0e*cf0+49*h6*f0e*sf0+105/2*h3*cf0+...
%         225/2*h4*sf0-105/2*h2*sf0/f0e)*f14/f0e5;
%     
%     %ook nog doen voor eerste en tweede correctie
%     
%     pue=u0e;
%     pve=v0e;
%     puep=u0pe;
%     pvep=v0pe;
    
    
end

 if forward
        T(:,:,i)=[pu, pv ; pup, pvp];
 else
        T(:,:,i)=[pvp, -pv ; -pup, pu];
 end
end