function varargout = schrod(varargin)
%SCHROD Schrodinger class constructor
% sch = schrod(V,min,max)
% sch = schrod(V,min,max,a0,b0,a1,b1)

switch nargin
case 0
    % if no input arguments, create a default object
   sch.V = 'x';
   sch.min = 0;
   sch.max = pi;
   sch.a0 = 1;
   sch.b0 = 0;
   sch.a1 = 1;
   sch.b1 = 0;
   sch = class(sch,'schrod');
case 1    %copy constructor
 if isa(varargin{1},'schrod')
     sch = varargin{1};
 else
     error('schrod-constructor: Input argument is not a schrodinger object')
 end
case {3,7,8}
    if ~isa(varargin{1},'char')
         error('schrod-constructor: arg1: You need to pass the function as string');  
    end
    if ~isa(varargin{2}, 'double') || ~isa(varargin{3}, 'double')
      error('schrod-constructor: arg2-arg3: You need to pass double-precision floating-point numbers');  
    end
    v= strrep([varargin{1} '+0*x'],'^','.^'); % makes it possible to do V(X) where X is a vector
    v= strrep(v,'*','.*');
    v= strrep(v,'/','./');
    v=strrep(v,'..','.');
    sch.V = inline(v);    %inline function
    sch.min = varargin{2};
    sch.max = varargin{3};
    if(nargin==3) 
            sch.a0 = 1;
            sch.b0 = 0;
            sch.a1 = 1;
            sch.b1 = 0;
   else
           if (~isa(varargin{4}, 'double')...
                || ~isa(varargin{5}, 'double')...
                || ~isa(varargin{6}, 'double')...
                || ~isa(varargin{7}, 'double'))
              error('schrod-constructor: arg4-arg7: You need to pass double-precision floating-point numbers');  
            end
            sch.a0 = varargin{4};
            sch.b0 = varargin{5};
            sch.a1 = varargin{6};
            sch.b1 = varargin{7};    
            if nargin ==8  
                if ~(isa(varargin{8},'char') || isa(varargin{8}, 'sym'))
                  error('schrod-constructor: arg8: You need to pass the unknown variable as character or as sym '); 
                end
                sch.V=inline(char(subs(v,sym(varargin{8}),sym('x'))));
            end    
    end  
    sch = class(sch,'schrod');
otherwise
    error('Wrong number of input arguments')
end
varargout{1}=sch;
if nargin >3
 if sch.min >= sch.max
    error('Error!!! : INVALID INTEGRATION INTERVAL')
 end
 if (abs(sch.a0)+abs(sch.b0) == 0) || (abs(sch.a1)+abs(sch.b1) == 0)
    error('Error!!! : INVALID BOUNDARY CONDITION')
 end
end