%Mathieu equation y''=(2*cos(2*x)-E)y on [0,pi]

disp('**********************************')
disp('*           Mathieu Equation     *')
disp('**********************************')

%First we specify the problem:
r=schrod('2*cos(2*x)',0,pi,1,0,1,0);
%The schrod constructor is used, the first argument is a string
%representing the potential function, the second and third argument are
%the two endpoints of the integration interval and the last arguments
%specify the boundary conditions

%We want to solve the problem with the LPM[4,2] method (which is of order 10).
%initialise the LP-method:
lp=lpm10(r,1e-10);
%We have chosen the tolerance to be 1e-10.

%The following command makes a plot of the partition and the reference
%potential:
plot_partition(lp);

bp=get(lp,'bp');
disp(['The number of intervals in the partition: ', num2str(length(bp.h))]);

%calculate the first eigenvalues:
disp('Calculating eigenvalues with indices between 0 and 20...')
t=cputime;
E=get_eigenvalues(lp,0,20,true); 
disp(' ');
disp('Eigenvalues with indices between 0 and 20:')
disp(['Time: ' num2str(cputime - t) ' sec']);
disp(sprintf('k \t  E_k \t\t\t\t  error \t  status'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e') '   ' num2str(E.status(i))])
end

%calculate the eigenvalues between 10 and 20:
E=get_eigenvalues(lp,10,20,false); 
%note that the partition has not to be calculated again, the same partition
%is still used.
disp(' ');
disp('Eigenvalues between 10 and 20:')
disp(['Time: ' num2str(cputime - t) ' sec']);
disp(sprintf('k \t  E_k \t\t\t\t  error \t  status'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e') '   ' num2str(E.status(i))])
end


