%Paine equation y''=(1/(x+0.1)^2-E)y on [0,pi]

disp('********************************')
disp('*           Paine Equation     *')
disp('********************************')

%First we specify the problem:
r=schrod('1/(x+0.1)^2',0,pi,1,0,1,0);
%The schrod constructor is used, the first argument is a string
%representing the potential function, the second and third argument are
%the two endpoints of the integration interval and the last arguments
%specify the boundary conditions

%We want to solve the problem with the LPM[4,2] method (which is of order 10).
%initialise the LP-method:
lp=lpm10(r,1e-10);
%We have chosen the tolerance to be 1e-10
plot_partition(lp);

bp=get(lp,'bp');
disp(['The number of intervals in the partition: ', num2str(length(bp.h))]);

%calculate the first 11 eigenvalues:
disp('Calculating eigenvalues with indices between 0 and 10...')
t=cputime;
E=get_eigenvalues(lp,0,10,true); 
disp(' ');
disp('Eigenvalues with indices between 0 and 10:')
disp(['Time: ' num2str(cputime - t) ' sec']);
disp(sprintf('k \t  E_k \t\t\t\t  error \t  status'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e') '   ' num2str(E.status(i))])
end



